//+------------------------------------------------------------------+
//|                                                 Sova_Signals.mq5 |
//|                        Copyright 2013, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2013, MetaQuotes Software Corp."
#property link      "http://www.mql5.com"
#property version   "1.00"
//+------------------------------------------------------------------+
//| Include                                                          |
//+------------------------------------------------------------------+
#include <Expert\Expert.mqh>
//--- available signals
#include <Expert\Signal\SignalAO.mqh>
#include <Expert\Signal\SignalMACD.mqh>
#include <Expert\Signal\SignalMA.mqh>
#include <Expert\Signal\SignalSAR.mqh>
#include <Expert\Signal\SignalStoch.mqh>
#include <Expert\Signal\SignalTRIX.mqh>
#include <Expert\Signal\SignalITF.mqh>
//--- available trailing
#include <Expert\Trailing\TrailingFixedPips.mqh>
//--- available money management
#include <Expert\Money\MoneyFixedLot.mqh>
#include <Indicators\TimeSeries.mqh>   //---  
//+------------------------------------------------------------------+
//| Inputs                                                           |
//+------------------------------------------------------------------+
//--- inputs for expert
input string             Expert_Title                  ="Sova_Signals"; // Document name
ulong                    Expert_MagicNumber            =27821;          // 
bool                     Expert_EveryTick              =false;          // 
//--- inputs for main signal
input int                Signal_ThresholdOpen          =13;             // Signal threshold value to open [0...100]
input int                Signal_ThresholdClose         =23;             // Signal threshold value to close [0...100]
input double             Signal_PriceLevel             =-30.0;            // Price level to execute a deal
input double             Signal_StopLevel              =46.0;           // Stop Loss level (in points)
input double             Signal_TakeLevel              =44.0;           // Take Profit level (in points)
input int                Signal_Expiration             =10;              // Expiration of pending orders (in bars)
input double             Signal_AO_Weight              =0.7;            // Awesome Oscillator M15 Weight [0...1.0]
input int                Signal_MACD_PeriodFast        =10;             // MACD(12,24,9,PRICE_CLOSE) M15 Period of fast EMA
input int                Signal_MACD_PeriodSlow        =15;             // MACD(12,24,9,PRICE_CLOSE) M15 Period of slow EMA
input int                Signal_MACD_PeriodSignal      =10;              // MACD(12,24,9,PRICE_CLOSE) M15 Period of averaging of difference
input ENUM_APPLIED_PRICE Signal_MACD_Applied           =PRICE_MEDIAN;    // MACD(12,24,9,PRICE_CLOSE) M15 Prices series
input double             Signal_MACD_Weight            =0.5;            // MACD(12,24,9,PRICE_CLOSE) M15 Weight [0...1.0]
input int                Signal_MA_PeriodMA            =26;             // Moving Average(12,0,...) M15 Period of averaging
input int                Signal_MA_Shift               =8;              // Moving Average(12,0,...) M15 Time shift
input ENUM_MA_METHOD     Signal_MA_Method              =MODE_SMMA;       // Moving Average(12,0,...) M15 Method of averaging
input ENUM_APPLIED_PRICE Signal_MA_Applied             =PRICE_MEDIAN;    // Moving Average(12,0,...) M15 Prices series
input double             Signal_MA_Weight              =0.8;            // Moving Average(12,0,...) M15 Weight [0...1.0]
input double             Signal_SAR_Step               =0.05;           // Parabolic SAR(0.02,0.2) M15 Speed increment
input double             Signal_SAR_Maximum            =0.7;            // Parabolic SAR(0.02,0.2) M15 Maximum rate
input double             Signal_SAR_Weight             =0.6;            // Parabolic SAR(0.02,0.2) M15 Weight [0...1.0]
input int                Signal_Stoch_PeriodK          =15;              // Stochastic(8,3,3,...) M15 K-period
input int                Signal_Stoch_PeriodD          =5;              // Stochastic(8,3,3,...) M15 D-period
input int                Signal_Stoch_PeriodSlow       =9;              // Stochastic(8,3,3,...) M15 Period of slowing
input ENUM_STO_PRICE     Signal_Stoch_Applied          =STO_LOWHIGH;    // Stochastic(8,3,3,...) M15 Prices to apply to
input double             Signal_Stoch_Weight           =0.2;            // Stochastic(8,3,3,...) M15 Weight [0...1.0]
input int                Signal_TriX_PeriodTriX        =27;             // Triple Exponential Average M15 Period of calculation
input ENUM_APPLIED_PRICE Signal_TriX_Applied           =PRICE_CLOSE;    // Triple Exponential Average M15 Prices series
input double             Signal_TriX_Weight            =0.3;            // Triple Exponential Average M15 Weight [0...1.0]
input int                Signal_ITF_GoodHourOfDay      =-1;             // IntradayTimeFilter(-1,0,-1,...) M15 Good hour
input int                Signal_ITF_BadHoursOfDay      =0;              // IntradayTimeFilter(-1,0,-1,...) M15 Bad hours (bit-map)
input int                Signal_ITF_GoodDayOfWeek      =-1;             // IntradayTimeFilter(-1,0,-1,...) M15 Good day of week
input int                Signal_ITF_BadDaysOfWeek      =0;              // IntradayTimeFilter(-1,0,-1,...) M15 Bad days of week (bit-map)
input double             Signal_ITF_Weight             =1.0;            // IntradayTimeFilter(-1,0,-1,...) M15 Weight [0...1.0]
//--- inputs for trailing
input int                Trailing_FixedPips_StopLevel  =44;             // Stop Loss trailing level (in points)
input int                Trailing_FixedPips_ProfitLevel=15;             // Take Profit trailing level (in points)
//--- inputs for money
input double             Money_FixLot_Percent          =10.0;           // Percent
input double             Money_FixLot_Lots             =1.0;            // Fixed volume

double StepIncrease = 20.0;
//+------------------------------------------------------------------+
//| Global expert object                                             |
//+------------------------------------------------------------------+
CExpert ExtExpert;
CExpertSignal m_signal;
   CExpertTrade     m_trade;                    // trading object
//+------------------------------------------------------------------+
//|                                  |
//+------------------------------------------------------------------+
CiOpen            myOpen;        //   CiOpen
CiHigh            myHigh;        //   CiHigh
CiLow             myLow;         //   CiLow
CiClose           myClose;       //   CiClose
//+------------------------------------------------------------------+
//|                                                         |
//+------------------------------------------------------------------+

//   
double
prc_c[], // Close (  )
prc_o[], // Open (  )
prc_h[], // High (  )
prc_l[]; // Low (  )

//+------------------------------------------------------------------+
//| Initialization function of the expert                            |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- Initializing expert
   if(!ExtExpert.Init("EURUSD",PERIOD_M5,Expert_EveryTick,Expert_MagicNumber))
     {
      //--- failed
      printf(__FUNCTION__+": error initializing expert");
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
//--- Creating signal
   CExpertSignal *signal=new CExpertSignal;
   if(signal==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating signal");
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
//---
   ExtExpert.InitSignal(signal);
   signal.ThresholdOpen(Signal_ThresholdOpen);
   signal.ThresholdClose(Signal_ThresholdClose);
   signal.PriceLevel(Signal_PriceLevel);
   signal.StopLevel(Signal_StopLevel);
   signal.TakeLevel(Signal_TakeLevel);
   signal.Expiration(Signal_Expiration);
//--- Creating filter CSignalAO
   CSignalAO *filter0=new CSignalAO;
   if(filter0==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating filter0");
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
   signal.AddFilter(filter0);
//--- Set filter parameters
   filter0.Period(PERIOD_M15);
   filter0.Weight(Signal_AO_Weight);
//--- Creating filter CSignalMACD
   CSignalMACD *filter1=new CSignalMACD;
   if(filter1==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating filter1");
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
   signal.AddFilter(filter1);
//--- Set filter parameters
   filter1.Period(PERIOD_M15);
   filter1.PeriodFast(Signal_MACD_PeriodFast);
   filter1.PeriodSlow(Signal_MACD_PeriodSlow);
   filter1.PeriodSignal(Signal_MACD_PeriodSignal);
   filter1.Applied(Signal_MACD_Applied);
   filter1.Weight(Signal_MACD_Weight);
//--- Creating filter CSignalMA
   CSignalMA *filter2=new CSignalMA;
   if(filter2==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating filter2");
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
   signal.AddFilter(filter2);
//--- Set filter parameters
   filter2.Period(PERIOD_M15);
   filter2.PeriodMA(Signal_MA_PeriodMA);
   filter2.Shift(Signal_MA_Shift);
   filter2.Method(Signal_MA_Method);
   filter2.Applied(Signal_MA_Applied);
   filter2.Weight(Signal_MA_Weight);
//--- Creating filter CSignalSAR
   CSignalSAR *filter3=new CSignalSAR;
   if(filter3==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating filter3");
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
   signal.AddFilter(filter3);
//--- Set filter parameters
   filter3.Period(PERIOD_M15);
   filter3.Step(Signal_SAR_Step);
   filter3.Maximum(Signal_SAR_Maximum);
   filter3.Weight(Signal_SAR_Weight);
//--- Creating filter CSignalStoch
   CSignalStoch *filter4=new CSignalStoch;
   if(filter4==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating filter4");
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
   signal.AddFilter(filter4);
//--- Set filter parameters
   filter4.Period(PERIOD_M15);
   filter4.PeriodK(Signal_Stoch_PeriodK);
   filter4.PeriodD(Signal_Stoch_PeriodD);
   filter4.PeriodSlow(Signal_Stoch_PeriodSlow);
   filter4.Applied(Signal_Stoch_Applied);
   filter4.Weight(Signal_Stoch_Weight);
//--- Creating filter CSignalTriX
   CSignalTriX *filter5=new CSignalTriX;
   if(filter5==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating filter5");
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
   signal.AddFilter(filter5);
//--- Set filter parameters
   filter5.Period(PERIOD_M15);
   filter5.PeriodTriX(Signal_TriX_PeriodTriX);
   filter5.Applied(Signal_TriX_Applied);
   filter5.Weight(Signal_TriX_Weight);
//--- Creating filter CSignalITF
   CSignalITF *filter6=new CSignalITF;
   if(filter6==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating filter6");
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
   signal.AddFilter(filter6);
//--- Set filter parameters
   filter6.Period(PERIOD_M15);
   filter6.GoodHourOfDay(Signal_ITF_GoodHourOfDay);
   filter6.BadHoursOfDay(Signal_ITF_BadHoursOfDay);
   filter6.GoodDayOfWeek(Signal_ITF_GoodDayOfWeek);
   filter6.BadDaysOfWeek(Signal_ITF_BadDaysOfWeek);
   filter6.Weight(Signal_ITF_Weight);
//--- Creation of trailing object
   CTrailingFixedPips *trailing=new CTrailingFixedPips;
   if(trailing==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating trailing");
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
//--- Add trailing to expert (will be deleted automatically))
   if(!ExtExpert.InitTrailing(trailing))
     {
      //--- failed
      printf(__FUNCTION__+": error initializing trailing");
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
//--- Set trailing parameters
   trailing.StopLevel(Trailing_FixedPips_StopLevel);
   trailing.ProfitLevel(Trailing_FixedPips_ProfitLevel);
//--- Creation of money object
   CMoneyFixedLot *money=new CMoneyFixedLot;
   if(money==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating money");
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
//--- Add money to expert (will be deleted automatically))
   if(!ExtExpert.InitMoney(money))
     {
      //--- failed
      printf(__FUNCTION__+": error initializing money");
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
//--- Set money parameters
   money.Percent(Money_FixLot_Percent);
   money.Lots(Money_FixLot_Lots);
//--- Check all trading objects parameters
   if(!ExtExpert.ValidationSettings())
     {
      //--- failed
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
//--- Tuning of all necessary indicators
   if(!ExtExpert.InitIndicators())
     {
      //--- failed
      printf(__FUNCTION__+": error initializing indicators");
      ExtExpert.Deinit();
      return(INIT_FAILED);
     }
//--- ok
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
//| Deinitialization function of the expert                          |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
   ExtExpert.Deinit();
  }
//+------------------------------------------------------------------+
//| "Tick" event handler function                                    |
//+------------------------------------------------------------------+
void OnTick()
  {
   ExtExpert.OnTick();
   GetDataBars(); //   
   Piramiding();  
  }
//+------------------------------------------------------------------+
//| "Trade" event handler function                                   |
//+------------------------------------------------------------------+
void OnTrade()
  {
   ExtExpert.OnTrade();
   
  }
//+------------------------------------------------------------------+
//| "Timer" event handler function                                   |
//+------------------------------------------------------------------+
void OnTimer()
  {
   ExtExpert.OnTimer();
  }
//+------------------------------------------------------------------+
//   
//+------------------------------------------------------------------+
void Piramiding()
  {
   string comment="hello :)"; //   
   double
   tp=0.0, // Take Profit
   sl=0.0, // Stop Loss
   lot=0.0,//       
//---
// ,   
   signal_2=GetTradingSignal();
//---
//   

   
   int sym_digits=(int)SymbolInfoInteger(_Symbol,SYMBOL_DIGITS);               
   double sym_ask=NormalizeDouble(SymbolInfoDouble(_Symbol,SYMBOL_ASK),sym_digits);                       
   double sym_bid=NormalizeDouble(SymbolInfoDouble(_Symbol,SYMBOL_BID),sym_digits);                       
//---
   double posit_sl=PositionGetDouble(POSITION_SL); //  Stop Loss  
   double posit_tp=PositionGetDouble(POSITION_TP); //  Take Profit  
//---

      //---
      //       
      //   ,    
      if(signal_2==1)
        {
      ExtExpert.OpenShort(sym_ask,posit_sl,posit_tp);
         return;
        }
      else if(signal_2==0)
        {
      ExtExpert.OpenLong(sym_bid,posit_sl,posit_tp);
         return;
        }
//---
   return;
  }//+------------------------------------------------------------------+
//|                                         |
//+------------------------------------------------------------------+
int GetTradingSignal()
  {
   bool posit_exist=PositionSelect(_Symbol);
   if(!posit_exist) //   
     {
      return(3);
     }
//---
   if(posit_exist) //   
     {
      uint posit_type=(ENUM_POSITION_TYPE)PositionGetInteger(POSITION_TYPE);//   
      double last_price=GetPriceLastDealCurrPos();//    
      //---
         if(posit_type==POSITION_TYPE_SELL && 
            prc_c[1]<last_price-DgtMlt(StepIncrease*_Point)) { return(1); }
         //---
         //---
         if(posit_type==POSITION_TYPE_BUY && 
            prc_c[1]>last_price+DgtMlt(StepIncrease*_Point)) { return(0); }
      //---
     }
//---
//   (3)
   return(3);
  }
//+------------------------------------------------------------------+
//|                       |
//+------------------------------------------------------------------+
double GetPriceLastDealCurrPos()
  {
   int total=0; //      
   ulong ticket=0; //  
   string deal_symbol=""; //   
   double deal_price=0.0; // 
   datetime posit_time=(datetime)PositionGetInteger(POSITION_TIME);
//---
//    
   if(HistorySelect(posit_time,TimeCurrent()))
     {
      //      
      total=HistoryDealsTotal();
      //---
      //         
      //       
      for(int i=total-1; i>=0; i--)
        {
         //         , ...
         if((ticket=HistoryDealGetTicket(i))>0)
           {
            deal_symbol=HistoryDealGetString(ticket,DEAL_SYMBOL); //   
            deal_price=HistoryDealGetDouble(ticket,DEAL_PRICE); //   
            //---
            //       ,  
            if(deal_symbol==_Symbol) { break; }
           }
        }
     }
//---
   return(deal_price);
  }

//+------------------------------------------------------------------+
//|        (int)             |
//+------------------------------------------------------------------+
int DgtMlt(int value)
  {
   int sym_digits=(int)SymbolInfoInteger(_Symbol,SYMBOL_DIGITS);               
   if(sym_digits==3 || sym_digits==5) { return(value*=10); } else { return(value); }
  }
//+------------------------------------------------------------------+
//|        (double)          |
//+------------------------------------------------------------------+
double DgtMlt(double value)
  {
   int sym_digits=(int)SymbolInfoInteger(_Symbol,SYMBOL_DIGITS);               
   if(sym_digits==3 || sym_digits==5) { return(value*=10); } else { return(value); }
  }
//+------------------------------------------------------------------+
//|                                             |
//+------------------------------------------------------------------+
void GetDataBars()
  {
//     (... 3 2 1 0)
   ArraySetAsSeries(prc_c,true);
   ArraySetAsSeries(prc_o,true);
   ArraySetAsSeries(prc_h,true);
   ArraySetAsSeries(prc_l,true);
//---
//    
//    ,  
//    
   if(CopyClose(_Symbol,_Period,0,3,prc_c)<3)
     {
      Print("    ");
     }
//---
//    
//    ,  
//    
   if(CopyOpen(_Symbol,_Period,0,3,prc_o)<3)
     {
      Print("    ");
     }
//---
//    
//    ,  
//    
   if(CopyHigh(_Symbol,_Period,0,3,prc_h)<3)
     {
      Print("    ");
     }
//---
//    
//    ,  
//    
   if(CopyLow(_Symbol,_Period,0,3,prc_l)<3)
     {
      Print("    ");
     }
  }